# eco_aprs_weather

- [eco_aprs_weather](#eco-aprs-weather)
    + [About](#about)
      - [Architecture Diagram](#architecture-diagram)
    + [Supported Devices](#supported-devices)
      - [Supported Weather Station Gateway Devices](#supported-weather-station-gateway-devices)
      - [Supported Weather Station Probes](#supported-weather-station-probes)
    + [Software Requirements](#software-requirements)
    + [Install Directions](#install-directions)
      - [Install for Debian/Ubuntu (deb) (For Amateurs)](#install-for-debian-ubuntu--deb---for-amateurs-)
      - [Install for Redhat (rpm) (For Amateurs)](#install-for-redhat--rpm---for-amateurs-)
      - [Install for FreeBSD (For Amateurs)](#install-for-freebsd--for-amateurs-)
      - [Install for Mac OS X (For Amateurs)](#install-for-mac-os-x--for-amateurs-)
      - [Simple Tarball Install (For Amateurs)](#simple-tarball-install--for-amateurs-)
      - [Install Using Python / Pip (For Intermediates)](#install-using-python---pip--for-intermediates-)
      - [Install Using Docker (For Experts Only)](#install-using-docker--for-experts-only-)
    + [Starting the `eco_aprs_weather` service](#starting-the--eco-aprs-weather--service)
    + [Stopping the `eco_aprs_weather` service](#stopping-the--eco-aprs-weather--service)
    + [Ecowitt App Configuration](#ecowitt-app-configuration)
    + [To confirm weather station telemtry is being recieved by the bridge](#to-confirm-weather-station-telemtry-is-being-recieved-by-the-bridge)
    + [Configuring your APRS software to send WX [Weather] Beacons!](#configuring-your-aprs-software-to-send-wx--weather--beacons-)
      - [Configuring Direwolf](#configuring-direwolf)
      - [Other APRS Software](#other-aprs-software)
    + Supplemental Documentation
      - [Config File Options](https://github.com/jondkelley/eco_aprs_weather/blob/master/docs/configuration.md)
      - [Troubleshooting Help](https://github.com/jondkelley/eco_aprs_weather/blob/master/docs/troubleshooting.md)
      - [Web/API Route Descriptions](https://github.com/jondkelley/eco_aprs_weather/blob/master/docs/webapi_spec.md)
    + Architectural References
      - [wxnow.txt system format](https://github.com/jondkelley/eco_aprs_weather/blob/master/docs/architectural_references/wxnow_txt.md)
      - [APRS Compressed Telemtry](https://github.com/jondkelley/eco_aprs_weather/blob/master/docs/architectural_references/aprs_compressed_telemetry.txt)
      - [APRS Telemetry](https://github.com/jondkelley/eco_aprs_weather/blob/master/docs/architectural_references/aprs_telemetry.md)
- [Special Thanks To:](#special-thanks-to-)


### About

The `eco_aprs_weather` software is a bridge to allow ECOWITT Weather Stations to integrate with APRS weather telemetry for amateur radio. 

This software integrates directly with ECOWITT GW brand weather stations.

Here is a possible architecture implementation of this software [the blue box(es)!]:

#### Architecture Diagram

![the actual reference architecture diagram](https://github.com/jondkelley/eco_aprs_weather/blob/master/images/architecture.drawio.png)

*Above: You would buy ECOWITT gateway + sensors [not shown in diagram], setup the block diagram components in white, and finally setup this software in the blue block diagam. The diagram depicts running on Raspberry Pi, but this software can run on an old PC, on Docker, or up in a cloud server; this software should work with many deployment methods.*

The architecture described in "layman's terms":

This software sits collecting weather metrics from your ECOWITT branded weather station at intervals you set in the ECOWITT app. You can configure your APRS Igate / RF software to periodically use the `wxnow.txt` weather beacons generated by `eco_aprs_weather` (this app.) The [wxnow.txt](https://github.com/jondkelley/eco_aprs_weather/blob/master/docs/architectural_references/wxnow_txt.md) format is used by popular APRS software to send radio weather telemetry packets. This software emits `wxnow.txt` directly over HTTP, and can be saved locally if required.

### Supported Devices

#### Supported Weather Station Gateway Devices

This software should work with ECOWITT Weather station gateways models like:

* ECOWITT GW1004
* ECOWITT GW1100
* ECOWITT GW1102 

Other ECOWITT Gateway models may be supported as long as your ECOWITT device settings can output to a customized HTTP server in "ECOWITT" station format. See the *Ecowitt App Configuration* section for configuration details in the ECOWITT phone app (which works for the models above)!

*NOTE 1: If you know of other compatible ECOWITT devices, I am good on QRZ @ N5IPT, let me know!!!*

#### Supported Weather Station Probes

This software supports:

* ECOWITT WH31 outdoor temperature&humidity sensor
* ECOWITT WH32 outdoor temperature&humidity sensor
* ECOWITT WH40 wireless self-emptying rain gauge sensor
* ECOWITT WS68 wireless solar powered anemometer

### Software Requirements

* Python 3.2 or greater.
* Memory: 1 GB minimum RAM suggested

This software works great on a Raspberry Pi along with the Direwolf Sofwarre TNC.

### Install Directions

#### Install for Debian/Ubuntu (deb) (For Amateurs)

tba [to be announced]

#### Install for Redhat (rpm) (For Amateurs)

tba [to be announced]

#### Install for FreeBSD (For Amateurs)

tba [to be announced]

#### Install for Mac OS X (For Amateurs)

tba [to be announced]

#### Simple Tarball Install (For Amateurs)

tba [to be announced]

#### Install Using Python / Pip (For Intermediates)

This software can be installed by Python Pip.

Make sure you have python and pip installed by running:

```
python3 --version
pip3 --version
```

If you're missing these, you'll need to install python, and python-pip.
If these commands worked, install eco_aprs_weather with:
```
sudo pip3 install eco_aprs_weather
````


#### Install Using Docker (For Experts Only)

If you want to run this with Docker continers, it is rather simple.

First, get [Docker Desktop](https://www.docker.com/products/docker-desktop/) installed 

Then you can run the application with:
```
docker run  -p 0.0.0.0:5000:5000/tcp jondkelley/eco_aprs_weather:latest
```

You should be able to visit `http://localhost:5000` to see the application!

If you want to do local software development, you can rebuild by using:
```
docker build . -t jondkelley/eco_aprs_weather:latest
```

### Starting the `eco_aprs_weather` service

You can simply run
```
eco_aprs_weather --screen
````

This will start the app in a screen session, afterwards you can now close the Terminal / CMD window.

*Note: If this command does not run, you need to make sure you have the following apps installed:*

* *screen*
* *curl*

### Stopping the `eco_aprs_weather` service

You can simply run
```
eco_aprs_weather --kill
````

*NOTE: This is only known to work on Linux based environments, or Windows Subsystem for Linux.*

### Ecowitt App Configuration

Once you have the service started for `Eco_APRS_Weather` you need to determine the IP address of your device running `eco_aprs_weather`.
The IP of this device is used to point your weather station reports to it.

Within your Ecowitt phone app:

1. Click the elipsis (`...`) symbol in the top right (of the screen showing your WX station live metrics)
2. In dropdown, select "Others"
3. In the "Edit Gateway" screen click "DIY Upload Servers" button
4. In the "DIY Upload Servers" screen click "Customized" (Globe Icon)
   * In the `"Server / Host Name"` field enter the IP address of the computer running this software
   * In the `"Port"` field make sure it is `"5000"`
   * In the `"Upload Interval"` field select the fastest interval
   * Click save.
5. Hit `<` (top right) to return to previous screens
6. WX station should start sending metrics to the bridge in the next few minutes!

### To confirm weather station telemtry is being recieved by the bridge

Visit `http://<IP ADDRESS OF THIS SOFTWARE>:5000/sensor/overview` and verify you have station data coming in.

You may have to wait 5-10 minutes and refresh this page periodically, sometimes it takes a while for Ecowitt device settings to take effect.

If you do not see station data, possible solutions involve:

* Check the Ecowitt WX Station configuration settings
* Ensure the path field says exactly "data/report", other values don't work.
* Ensure the computer running this software does not have any sort of Firewall enabled
* Verifying your network configuration and trying again

![overview pic](https://raw.githubusercontent.com/jondkelley/eco_aprs_weather/master/images/sensor_overview.png)


### Configuring your APRS software to send WX [Weather] Beacons!

This is the tricky part. This bridge supports any APRS software that accepts `wxnow.txt` weather format in Cumulus weather format.

My only experience is with [Direwolf](https://github.com/wb2osz/direwolf), the modern software replacement for the old 1980's style TNC.

#### Configuring Direwolf

First thing, get your Direwolf tested and working with normal `PBEACON` config statements to verify things are up and running. That's beyond the scope of this README.

Once Direwolf is confirmed working, you can comment out your old `PBEACON` with a `#` chaacter, and try a *weather telemtry beacon* like this:

```
PBEACON sendto=IG delay=0:30 every=13 lat=1.303690 long=-1.628359 SYMBOL="weather station" COMMENTCMD="curl -s http://127.0.0.1:5000/wxnow.txt | tail -1"
```

*Note 1: Be sure to change the `lat` `long` fields to match your physical coordinates!*

*Note 2: Change 127.0.0.1 to match the IP address of your eco_aprs_weather bridge IP!*

*Note 3: If your running this software on the same Raspberry Pi with Direwolf, you can leave 127.0.0.1 alone.*

This will send your weather reports straight the internet's APRS-IS backend service. It's what most people do. You can add an additional `PBEACON` line and change `sendto=IG` to `sendto=0` to send to the first radio channel in direwolf.

This would look like:
```
PBEACON sendto=IG delay=0:30 every=13 lat=1.303690 long=-1.628359 SYMBOL="weather station" COMMENTCMD="curl -s http://127.0.0.1:5000/wxnow.txt | tail -1"
PBEACON sendto=0 delay=0:30 every=13 lat=1.303690 long=-1.628359 SYMBOL="weather station" COMMENTCMD="curl -s http://127.0.0.1:5000/wxnow.txt | tail -1"
```

Having both the above `PBEACON` statements causes transmission on both `radio port 0` and `IG` (igate), sending over local RF as well as APRS-IS. Local RF packets can be received by APRS/WX-aware radios like Kenwood's or maybe some Yaesu's.


#### Other APRS Software

I don't know what other APRS software accepts `wxnow.txt` and can report telemetry. That's where you come in! I need help, I don't know *everything* APRS.

Here's a *PRO TIP* how you can save `wxnow.txt` to the disk of any popular Linux/UNIX based system (such as Raspberry Pi):

```
echo "*/5 * * * * root curl -s http://127.0.0.1:5000/wxnow.txt | tail -1" >> /wxnow.txt
```


Contact me thru QRZ (N5IPT) and write up some instructions and I can post them here. You will recieve credit by default unless you ask not to be mentioned in this README.

If you know how to use Github and can make a [Pull Request](https://github.com/jondkelley/eco_aprs_weather/pulls), even better!


# Special Thanks To:

* Ben (N5AMD) for donating hardware, URI, Raspberry Pi 1 [2011], and a 2meter rig to prototype this software with.

